#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from dotty_dict import dotty


class TestListInDotty(unittest.TestCase):
    def setUp(self):
        self.dot = dotty({
            'field1': 'Value of F1',
            'field2': 'Value of F2',
            'field3': [
                {
                    'subfield1': 'Value of subfield1 (item 0)',
                    'subfield2': 'Value of subfield2 (item 0)'
                },
                {
                    'subfield1': 'Value of subfield1 (item 1)',
                    'subfield2': 'Value of subfield2 (item 1)'
                },
            ],
            'field4': 'Not wanted',
            'field5': [
                {
                    'subfield1': [{'subsubfield': 'Value of sub subfield (item 0)'}]
                }
            ],
            'field6': ['a', 'b']
        })

    def test_root_level_list_element(self):
        self.assertEqual(self.dot['field6.0'], 'a')

    def test_access_subfield1_of_field3(self):
        self.assertEqual(self.dot['field3.0.subfield1'], 'Value of subfield1 (item 0)')
        self.assertEqual(self.dot['field3.1.subfield1'], 'Value of subfield1 (item 1)')

    def test_access_sub_sub_field(self):
        self.assertEqual(self.dot['field5.0.subfield1.0.subsubfield'], 'Value of sub subfield (item 0)')

    def test_access_multidimensional_lists(self):
        dot = dotty({
            'field': [
                [{'subfield': 'Value of subfield (item 0,0)'}],
                [{'subfield': 'Value of subfield (item 0,1)'}]
            ]
        })
        self.assertEqual(dot['field.1.0.subfield'], 'Value of subfield (item 0,1)')

    def test_dotty_contains_subfield_of_field(self):
        self.assertIn('field3.0.subfield2', self.dot)

    def test_dotty_not_contains_out_of_range_subfield(self):
        self.assertNotIn('field3.3.subfield1', self.dot)

    def test_assert_key_error_if_index_is_not_integer(self):
        with self.assertRaises(KeyError):
            val = self.dot['field3.subfield1']  # noqa

    def test_assert_index_error_if_index_is_out_of_range(self):
        with self.assertRaises(IndexError):
            val = self.dot['field3.4.subfield1']  # noqa

    def test_assert_get_returns_default_if_index_is_out_of_range(self):
        val = self.dot.get('field3.4.subfield1')
        assert val is None

    def test_set_subfield_in_list(self):
        dot = dotty()

        dot['field.0.subfield'] = 'Value of subfield (item 0)'
        dot['field.1.subfield'] = 'Value of subfield (item 1)'
        dot['field.1.subfield2'] = 'Value of subfield2 (item 1)'

        self.assertDictEqual(dot.to_dict(), {
            'field': [
                {'subfield': 'Value of subfield (item 0)'},
                {'subfield': 'Value of subfield (item 1)', 'subfield2': 'Value of subfield2 (item 1)'}
            ],
        })

    def test_update_subfield_in_list(self):
        dot = dotty({
            'field': [
                {'subfield': 'Value of subfield (item 0)'},
                {'subfield': 'Value of subfield (item 1)', 'subfield2': 'Value of subfield2 (item 1)'}
            ],
        })

        dot['field.0.subfield'] = 'updated value'

        self.assertDictEqual(dot.to_dict(), {
            'field': [
                {'subfield': 'updated value'},
                {'subfield': 'Value of subfield (item 1)', 'subfield2': 'Value of subfield2 (item 1)'}
            ],
        })

    def test_delete_subfield(self):
        dot = dotty({'field': [
            {
                'subfield1': 'Value of subfield1 (item 0)',
                'subfield2': 'Value of subfield2 (item 0)'
            },
        ]})

        del dot['field.0.subfield2']

        self.assertDictEqual(dot.to_dict(), {'field': [
            {'subfield1': 'Value of subfield1 (item 0)'},
        ]})

    def test_list_as_return_value(self):
        dot = dotty({
            'field': [
                'list_field0',
                'list_field1'
            ]
        })

        self.assertEqual(dot['field.0'], 'list_field0')
        self.assertEqual(dot['field.1'], 'list_field1')
        self.assertTrue('field.0' in dot)
        self.assertTrue('field.1' in dot)
        self.assertFalse('field.2' in dot)


class TestMultipleSelectList(unittest.TestCase):
    def setUp(self):
        self.dot = dotty({
            'field1': [
                {
                    "subfield1": "value01",
                    "subfield2": "value02"
                },
                {
                    "subfield1": "value11",
                    "subfield2": "value12"
                },
                {
                    "subfield1": "value21",
                    "subfield2": "value22"
                },
                {
                    "subfield1": "value31",
                    "subfield2": "value32"
                }
            ],
            "field2": [
                {
                    "subfield1": [
                        {
                            "nestedsubfield1": "nestedvalue001",
                            "nestedsubfield2": "nestedvalue002"
                        },
                        {
                            "nestedsubfield1": "nestedvalue011",
                            "nestedsubfield2": "nestedvalue012"
                        }
                    ]
                },
                {
                    "subfield1": [
                        {
                            "nestedsubfield1": "nestedvalue101",
                            "nestedsubfield2": "nestedvalue102"
                        },
                        {
                            "nestedsubfield1": "nestedvalue111",
                            "nestedsubfield2": "nestedvalue112"
                        }
                    ]
                }
            ]
        })

    def test_whole_shallow_multiple_list(self):
        expected_list = ['value01', 'value11', 'value21', 'value31']
        self.assertListEqual(self.dot['field1.:.subfield1'], expected_list)

    def test_whole_nested_multiple_list(self):
        expected_list = [['nestedvalue001', 'nestedvalue011'],
                         ['nestedvalue101', 'nestedvalue111']]
        self.assertListEqual(self.dot['field2.:.subfield1.:.nestedsubfield1'], expected_list)

    def test_left_side_slice(self):
        expected_list = ['value21', 'value31']
        self.assertListEqual(self.dot['field1.2:.subfield1'], expected_list)

    def test_right_side_slice(self):
        expected_list = ['value01', 'value11']
        self.assertListEqual(self.dot['field1.:2.subfield1'], expected_list)

    def test_both_side_slice(self):
        expected_list = ['value11', 'value21']
        self.assertListEqual(self.dot['field1.1:3.subfield1'], expected_list)

    def test_step_slice(self):
        expected_list = ['value01', 'value21']
        self.assertListEqual(self.dot['field1.::2.subfield1'], expected_list)

    def test_return_whole_list(self):
        expected_list = [
            {
                "subfield1": "value01",
                "subfield2": "value02"
            },
            {
                "subfield1": "value11",
                "subfield2": "value12"
            },
            {
                "subfield1": "value21",
                "subfield2": "value22"
            },
            {
                "subfield1": "value31",
                "subfield2": "value32"
            }
        ]
        self.assertListEqual(self.dot['field1.:'], expected_list)


class TestNoList(unittest.TestCase):
    def setUp(self):
        self.dot = dotty({
            'field1': {
                '1': 'value1',
                '2': {
                    'subfield1': 'value2',
                    'subfield2': 'value3'
                },
                ':': 'value4',
                '2:': 'value5',
                'key': 'value6'
            },
        }, no_list=True)

    def test_simple_index(self):
        self.assertEqual(self.dot['field1.1'], 'value1')

    def test_whole_slice_index(self):
        self.assertEqual(self.dot['field1.:'], 'value4')

    def test_limit_slice_index(self):
        self.assertEqual(self.dot['field1.2:'], 'value5')

    def test_normal_key(self):
        self.assertEqual(self.dot['field1.key'], 'value6')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from dotty_dict import Dotty, dotty


class TestDottyPrivateMembers(unittest.TestCase):
    def test_split_separator(self):
        dot = dotty()
        result = dot._split('chain.of.keys')
        self.assertListEqual(result, ['chain', 'of', 'keys'])

    def test_split_with_custom_separator(self):
        dot = Dotty({}, separator='#', esc_char='\\')
        result = dot._split('chain#of#keys')
        self.assertListEqual(result, ['chain', 'of', 'keys'])


class TestDottyPublicMembers(unittest.TestCase):
    def test_to_dict(self):
        plain_dict = {'very': {'deeply': {'nested': {'thing': 'spam'}}}}
        dot = dotty(plain_dict)
        self.assertIsInstance(dot.to_dict(), dict)
        self.assertEqual(sorted(dot.to_dict().items()), sorted(plain_dict.items()))

    def test_nested_dotty_object_to_dict(self):
        expected_dict = {'hello': {'world': 1}, 'nested': {'dotty': {'wazaa': 3}}}
        top_dot = dotty({'hello': {'world': 1}})
        nested_dot = dotty({'wazaa': 3})
        top_dot['nested.dotty'] = nested_dot
        self.assertDictEqual(top_dot.to_dict(), expected_dict)

    def test_nested_dotty_in_list_to_dict(self):
        expected_dict = {'testlist': [{'dot1': 1}, {'dot2': 2}]}
        dot_list = [dotty({'dot1': 1}), dotty({'dot2': 2})]
        top_dot = dotty({'testlist': dot_list})
        self.assertDictEqual(top_dot.to_dict(), expected_dict)


class TestDictSpecificMethods(unittest.TestCase):
    def setUp(self):
        self.dot = dotty({
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_access_keys(self):
        keys = list(sorted(self.dot.keys()))
        self.assertListEqual(keys, ['deep', 'flat_key'])

    def test_access_keys_from_deeply_nested_structure(self):
        keys = list(sorted(self.dot['deep.deeper'].keys()))
        self.assertListEqual(keys, ['ridiculous', 'secret'])

    def test_get_value_without_default(self):
        result = self.dot.get('deep.nested')
        self.assertEqual(result, 12)

    def test_get_value_with_default(self):
        result = self.dot.get('deep.other', False)
        self.assertFalse(result)

    def test_return_dotty_length(self):
        self.assertEqual(len(self.dot), 2)

    def test_pop_from_dotty_flat(self):
        result = self.dot.pop('flat_key')
        self.assertEqual(result, 'flat value')
        self.assertDictEqual(self.dot._data, {
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_pop_with_default_value(self):
        result = self.dot.pop('not_existing', 'abcd')
        self.assertEqual(result, 'abcd')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_pop_nested_key(self):
        result = self.dot.pop('deep.nested')
        self.assertEqual(result, 12)
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_pop_nested_key_with_default_value(self):
        result = self.dot.pop('deep.deeper.not_existing', 'abcd')
        self.assertEqual(result, 'abcd')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_setdefault_flat_not_existing(self):
        result = self.dot.setdefault('next_flat', 'new default value')
        self.assertEqual(result, 'new default value')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'next_flat': 'new default value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_setdefault_flat_existing(self):
        self.dot['next_flat'] = 'original value'
        result = self.dot.setdefault('next_flat', 'new default value')
        self.assertEqual(result, 'original value')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'next_flat': 'original value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_setdefault_nested_key_not_existing(self):
        result = self.dot.setdefault('deep.deeper.next_key', 'new default value')
        self.assertEqual(result, 'new default value')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'next_key': 'new default value',
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_setdefault_nested_key_existing(self):
        self.dot['deep.deeper.next_key'] = 'original value'
        result = self.dot.setdefault('deep.deeper.next_key', 'new default value')
        self.assertEqual(result, 'original value')
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'next_key': 'original value',
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_copy(self):
        first = dotty({'a': 1, 'b': 2})
        second = first.copy()

        self.assertIsInstance(second, Dotty)
        self.assertEqual(first, second)
        self.assertIsNot(first, second)
        self.assertIsNot(first._data, second._data)

    def test_fromkeys(self):
        dot = dotty().fromkeys({'a', 'b', 'c'}, value=10)
        self.assertDictEqual(dot.to_dict(), {'a': 10, 'b': 10, 'c': 10})
        self.assertIsInstance(dot, Dotty)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from example import fetch_all_examples_for_testing

__author__ = 'Paweł Zadrożny'
__copyright__ = 'Copyright (c) 2018, Pawelzny'


def test_examples():
    """Entry point for examples functions.

    All examples are stores in example module
    in form of functions with assertions.

    Any exception raised in example will break tests.
    """
    for func in fetch_all_examples_for_testing():
        func()



# -*- coding: utf-8 -*-
from dotty_dict.dotty_dict import Dotty, dotty

__author__ = 'Paweł Zadrożny'
__copyright__ = 'Copyright (c) 2017, Paweł Zadrożny'
__email__ = 'pawel.zny@gmail.com'
__version__ = '1.3.0'
__all__ = ['Dotty', 'dotty']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from collections.abc import Mapping
except ImportError:
    from collections import Mapping

from functools import lru_cache
import json

__author__ = 'Paweł Zadrożny'
__copyright__ = 'Copyright (c) 2017, Paweł Zadrożny'


def dotty(dictionary=None, no_list=False):
    """Factory function for Dotty class.

    Create Dotty wrapper around existing or new dictionary.

    :param dict dictionary: Any dictionary or dict-like object
    :param bool no_list: If set to True then numeric keys will NOT be converted to list indices
    :return: Dotty instance
    """
    if dictionary is None:
        dictionary = {}
    return Dotty(dictionary, separator='.', esc_char='\\', no_list=no_list)


class Dotty:
    """Dictionary and dict-like objects wrapper.

    Dotty wraps dictionary and provides proxy for quick accessing to deeply
    nested keys and values using dot notation.

    Dot notation can be customize in special cases. Let's say dot character
    has special meaning, and you want to use other character for accessing
    deep keys.

    Dotty does not copy original dictionary but it operates on it.
    All changes made in original dictionary are reflected in dotty wrapped dict
    and vice versa.

    :param dict dictionary: Any dictionary or dict-like object
    :param str separator: Character used to chain deep access.
    :param str esc_char: Escape character for separator.
    :param bool no_list: If set to True then numeric keys will NOT be converted to list indices
    """

    def __init__(self, dictionary, separator='.', esc_char='\\', no_list=False):
        if not isinstance(dictionary, (Mapping, dict)):
            raise AttributeError('Dictionary must be type of dict')
        else:
            self._data = dictionary
        self.separator = separator
        self.esc_char = esc_char
        self.no_list = no_list

    def __repr__(self):
        return 'Dotty(dictionary={}, separator={!r}, esc_char={!r})'.format(
            self._data, self.separator, self.esc_char)

    def __str__(self):
        return str(self._data)

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        try:
            return sorted(self._data.items()) == sorted(other.items())
        except AttributeError:
            return False

    def __len__(self):
        return len(self._data)

    def __getattr__(self, item):
        return getattr(self._data, item)

    def __contains__(self, item):
        def search_in(items, data):
            """Recursively search for deep key in dict.

            :param list items: List of dictionary keys
            :param data: Portion of dictionary to operate on
            :return bool: Predicate of key existence
            """
            it = items.pop(0)
            if it.isdigit():
                idx = int(it)
                if idx < len(data):
                    if items:
                        return search_in(items, data[idx])
                    else:
                        return data[idx]
                else:
                    return False

            if items and it in data:
                return search_in(items, data[it])
            return it in data

        return search_in(self._split(item), self._data)

    @staticmethod
    def _find_data_type(item, data):
        """This method returns item in datatype that exists in data dict.

        Method creates set of types present in dict keys
        and then iterates through them trying to convert item
        into one of types and check whether item under this type
        exists in dict keys. If yes then it'll return converted item.
        Otherwise item stays the same type as it was on entry.

        :param item: Item to convert to proper type
        :type item: any type

        :return: Converted or unchanged item
        :rtype: any type
        """
        data_types = set([type(i) for i in data.keys()])
        for t in data_types:
            try:
                if t(item) in data:
                    item = t(item)
                    return item
            except ValueError:
                pass
        return item

    @lru_cache(maxsize=32)
    def __getitem__(self, item):
        def get_from(items, data):
            """Recursively get value from dictionary deep key.

            :param list items: List of dictionary keys
            :param data: Portion of dictionary to operate on
            :return: Value from dictionary
            :raises KeyError: If key does not exist
            """
            it = items.pop(0)
            if isinstance(data, list) and it.isdigit() and not self.no_list:
                it = int(it)
            elif it not in data and isinstance(data, dict):
                it = self._find_data_type(it, data)
            elif isinstance(data, list) and ':' in it and not self.no_list:
                list_slice = slice(*map(lambda x: None if x == '' else int(x), it.split(':')))
                if items:
                    return [get_from(items.copy(), x) for x in  data[list_slice]]
                else:
                    return data[list_slice]
            try:
                data = data[it]
            except TypeError:
                raise KeyError("List index must be an integer, got {}".format(it))
            if items:
                return get_from(items, data)
            else:
                return data

        return get_from(self._split(item), self._data)

    def __setitem__(self, key, value):
        def set_to(items, data):
            """Recursively set value to dictionary deep key.

            :param list items: List of dictionary keys
            :param data: Portion of dictionary to operate on
            """
            it = items.pop(0)
            if items:

                if items[0].isdigit():
                    next_item = []
                else:
                    next_item = {}

                if it.isdigit():
                    it = int(it)
                    try:
                        if not data[it]:
                            data[it] = next_item
                    except IndexError:
                        self.set_list_index(data, it, next_item)
                    set_to(items, data[it])
                else:
                    if not data.get(it):
                        data[it] = next_item
                    set_to(items, data[it])

            else:
                if it.isdigit():
                    self.set_list_index(data, it, value)
                else:
                    data[it] = value

        set_to(self._split(key), self._data)

    @staticmethod
    def set_list_index(data, index, value):
        """Set value in list at specified index.
        All the values before target index should stay unchanged
        or be filled with None.
        :param data: List where value should be set
        :param index: String or Int of target index
        :param value: Target value to put under index
        """
        for _ in range(len(data), int(index) + 1):
            data.append(None)
        else:
            data[int(index)] = value

    def __delitem__(self, key):
        def del_key(items, data):
            """Recursively remove deep key from dict.

            :param list items: List of dictionary keys
            :param data: Portion of dictionary to operate on
            :raises KeyError: If key does not exist
            """
            it = items.pop(0)
            if it.isdigit():
                it = int(it)
            if items:
                del_key(items, data[it])
            else:
                del data[it]

        del_key(self._split(key), self._data)

    def copy(self):
        """Returns a shallow copy of dictionary wrapped in Dotty.

        :return: Dotty instance
        """
        return dotty(self._data.copy())

    @staticmethod
    def fromkeys(seq, value=None):
        """Create a new dictionary with keys from seq and values set to value.

        New created dictionary is wrapped in Dotty.

        :param seq: Sequence of elements which is to be used as keys for the new dictionary
        :param value: Value which is set to each element of the dictionary
        :return: Dotty instance
        """
        return dotty(dict.fromkeys(seq, value))

    def get(self, key, default=None):
        """Get value from deep key or default if key does not exist.

        This method match 1:1 with dict .get method except that it
        accepts deeply nested key with dot notation.

        :param str key: Single key or chain of keys
        :param Any default: Default value if deep key does not exist
        :return: Any or default value
        """
        try:
            return self.__getitem__(key)
        except (KeyError, IndexError):
            return default

    def pop(self, key, default=None):
        """Pop key from Dotty.

        This method match 1:1 with dict .pop method except that
        it accepts deeply nested key with dot notation.

        :param str key: Single key or chain of keys
        :param Any default: If default is provided will be returned
        :raises KeyError: If key does not exist and default has not been provided
        :return: Any or default value
        """

        def pop_from(items, data):
            it = items.pop(0)
            if it not in data:
                return default
            if items:
                data = data[it]
                return pop_from(items, data)
            else:
                return data.pop(it, default)

        return pop_from(self._split(key), self._data)

    def setdefault(self, key, default=None):
        """Get key value if exist otherwise set default value under given key
        and return its value.

        This method match 1:1 with dict .setdefault method except that
        it accepts deeply nested key with dot notation.

        :param str key: Single key or chain of keys
        :param Any default: Default value for not existing key
        :return: Value under given key or default
        """
        try:
            return self.__getitem__(key)
        except KeyError:
            self.__setitem__(key, default)
            return default

    def to_dict(self):
        """Return wrapped dictionary.

        This method does not copy wrapped dictionary.

        :return dict: Wrapped dictionary
        """
        return json.loads(self.to_json())

    def to_json(self):
        """Return wrapped dictionary as json string.

        This method does not copy wrapped dictionary.

        :return str: Wrapped dictionary as json string
        """
        return json.dumps(self._data, cls=DottyEncoder)


    def _split(self, key):
        """Split dot notated chain of keys.

        Works with custom separators and escape characters.

        :param str key: Single key or chain of keys
        :return list: List of keys
        """
        if not isinstance(key, str):
            return [key]
        esc_stamp = (self.esc_char + self.separator, '<#esc#>')
        skp_stamp = ('\\' + self.esc_char + self.separator, '<#skp#>' + self.separator)

        stamp_esc = ('<#esc#>', self.separator)
        stamp_skp = ('<#skp#>', self.esc_char)

        key = key.replace(*skp_stamp).replace(*esc_stamp)
        keys = key.split(self.separator)
        for i, k in enumerate(keys):
            keys[i] = k.replace(*stamp_esc).replace(*stamp_skp)

        return keys


class DottyEncoder(json.JSONEncoder):
    """Helper class for encoding of nested Dotty dicts into standard dict
    """
    def default(self, obj):
        """Return dict data of Dotty when possible or encode with standard format

        :param object: Input object

        :return: Serializable data
        """
        if hasattr(obj, '_data'):
            return obj._data
        else:
            return json.JSONEncoder.default(self, obj)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from dotty_dict import Dotty, dotty


class TestDottyValueAccess(unittest.TestCase):
    def setUp(self):
        self.dot = dotty({
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_access_flat_value(self):
        self.assertEqual(self.dot['flat_key'], 'flat value')

    # noinspection PyUnusedLocal
    def test_raise_key_error_if_key_does_not_exist(self):
        with self.assertRaises(KeyError):
            val = self.dot['not_existing']  # noqa

    def test_access_deep_nested_value(self):
        self.assertEqual(self.dot['deep.nested'], 12)

    def test_access_middle_nested_value(self):
        self.assertDictEqual(self.dot['deep.deeper.ridiculous'],
                             {'hell': 'is here'})

    def test_set_flat_value(self):
        self.dot['new_flat'] = 'super flat'
        self.assertIn('new_flat', self.dot)

    def test_set_deep_nested_value(self):
        self.dot['deep.new_key'] = 'new value'
        self.assertIn('new_key', self.dot['deep'])

    def test_set_new_deeply_nested_value(self):
        self.dot['other.chain.of.keys'] = True
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
            'other': {
                'chain': {
                    'of': {
                        'keys': True,
                    },
                },
            },
        })

    def test_dotty_has_flat_key(self):
        self.assertIn('flat_key', self.dot)

    def test_dotty_has_deeply_nested_key(self):
        self.assertIn('deep.nested', self.dot)

    def test_dotty_has_not_flat_key(self):
        self.assertNotIn('some_key', self.dot)

    def test_dotty_has_not_deeply_nested_key(self):
        self.assertNotIn('deep.other.chain', self.dot)

    def test_has_in(self):
        result = 'deep.deeper.secret' in self.dot
        self.assertTrue(result)

    def test_has_not_in(self):
        result = 'deep.other' in self.dot
        self.assertFalse(result)

    def test_delete_flat_key(self):
        del self.dot['flat_key']
        self.assertDictEqual(self.dot._data, {
            'deep': {
                'nested': 12,
                'deeper': {
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_delete_nested_key(self):
        del self.dot['deep.deeper.secret']
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_raise_key_error_on_delete_not_existing_key(self):
        with self.assertRaises(KeyError):
            del self.dot['deep.deeper.key']

    def test_set_value_with_escaped_separator(self):
        self.dot[r'deep.deeper.escaped\.dot_key'] = 'it works!'
        self.assertDictEqual(self.dot._data, {
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'escaped.dot_key': 'it works!',
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })

    def test_get_value_with_escaped_separator(self):
        dot = dotty({
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'escaped.dot_key': 'it works!',
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })
        result = dot[r'deep.deeper.escaped\.dot_key']
        self.assertEqual(result, 'it works!')

    def test_get_value_with_escaped_escape_separator(self):
        dot = dotty({
            'flat_key': 'flat value',
            'deep': {
                'nested': 12,
                'deeper': {
                    'escaped\\': {
                        'dot_key': 'it works!',
                    },
                    'secret': 'abcd',
                    'ridiculous': {
                        'hell': 'is here',
                    },
                },
            },
        })
        result = dot[r'deep.deeper.escaped\\.dot_key']
        self.assertEqual(result, 'it works!')

    def test_use_custom_separator_and_custom_escape_char(self):
        sep = ','
        esc = '$'
        dot = Dotty({}, separator=sep, esc_char=esc)
        dot['abcd,efg,hij'] = 'test'
        dot['abcd,efg$,hij'] = 'test2'
        dot[r'abcd,efg\$,hij'] = 'test3'
        self.assertDictEqual(dot._data, {
            'abcd': {
                'efg': {
                    'hij': 'test',
                },
                'efg,hij': 'test2',
                'efg$': {
                    'hij': 'test3',
                },
            },
        })

    def test_string_digit_key(self):
        dot = dotty({'field': {
            '1': 'one',
            '5': 'five'
        }})

        dict_one = dot['field.1']
        dict_five = dot['field.5']

        self.assertEqual(dict_one, 'one')
        self.assertEqual(dict_five, 'five')

    def test_integer_keys(self):
        dot = dotty({'field': {
            1: 'one',
            5: 'five'
        }})

        dict_one = dot['field.1']
        dict_five = dot['field.5']

        self.assertEqual(dict_one, 'one')
        self.assertEqual(dict_five, 'five')

    def test_data_gathering_with_int(self):
        dot = dotty({
            '2': 'string_value',
            2: 'int_value',
            'nested': {
                '2': 'nested_string_value',
                3: 'nested_int_value'
            }
        })

        dict_string = dot['2']
        dict_int = dot[2]
        nested_dict_string = dot['nested.2']
        nested_dict_int = dot['nested.3']

        self.assertEqual(dict_string, 'string_value')
        self.assertEqual(dict_int, 'int_value')
        self.assertEqual(nested_dict_string, 'nested_string_value')
        self.assertEqual(nested_dict_int, 'nested_int_value')

    def test_non_standard_key_types(self):
        dot = Dotty({
            3.3: 'float',
            True: 'bool',
            None: 'None',
            'nested': {
                4.4: 'nested_float'
            }
        }, separator=',')

        dict_float = dot[3.3]
        dict_bool = dot[True]
        dict_none = dot[None]
        nested_dict_float = dot['nested,4.4']
        self.assertEqual(dict_float, 'float')
        self.assertEqual(dict_bool, 'bool')
        self.assertEqual(dict_none, 'None')
        self.assertEqual(nested_dict_float, 'nested_float')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import subprocess

from setuptools import find_packages, setup
from setuptools.command.develop import develop
from setuptools.command.install import install
from setuptools.command.test import test

__author__ = 'Paweł Zadrożny'
__copyright__ = 'Copyright (c) 2018, Pawelzny'

with open('README.rst', 'r') as readme_file:
    readme = readme_file.read()


def get_version(*file_paths):
    """Retrieves the version from project/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


class PostDevelopCommand(develop):
    """Post-installation for development mode."""

    def run(self):
        subprocess.check_call(['pipenv', 'install', '--dev', '--deploy', '--system'])
        develop.run(self)


class TestCommand(test):
    """Run tests"""

    def run(self):
        subprocess.check_call(['pytest'])
        test.run(self)


setup(
    name='dotty_dict',
    version=get_version('dotty_dict', '__init__.py'),
    description="Dictionary wrapper for quick access to deeply nested keys.",
    long_description=readme,
    license="MIT license",
    author="Paweł Zadrożny @pawelzny",
    author_email='pawel.zny@gmail.com',
    url='https://github.com/pawelzny/dotty_dict',
    packages=find_packages(exclude=('tests', 'docs', 'bin', 'example')),
    package_dir={'dotty_dict': 'dotty_dict'},
    include_package_data=True,
    use_scm_version=True,
    install_requires=['setuptools_scm'],
    zip_safe=False,
    keywords='dot notation dict wrapper helper utils lib',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'License :: OSI Approved :: MIT License',
    ],
    cmdclass={
        'develop': PostDevelopCommand,
        'test': TestCommand,
    },
)



import unittest
from unittest.mock import MagicMock

from dotty_dict import dotty


class TestDottyCache(unittest.TestCase):

    def test_getitem_cache(self):
        dot = dotty()
        dot._data = MagicMock()
        for _ in range(10):
            dot.get('x.y.z')
        self.assertEqual(dot.__getitem__.cache_info().hits, 9)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from dotty_dict import dotty


class TestDottyBasics(unittest.TestCase):
    def test_create_empty_instance(self):
        dot = dotty()
        self.assertEqual(dot, {})

    def test_create_non_empty_instance(self):
        plain_dict = {'not': 'empty'}

        dot = dotty(plain_dict)
        self.assertEqual(dot, plain_dict)
        self.assertIsNot(dot, plain_dict)

    # noinspection PyTypeChecker
    def test_raise_attr_error_if_input_is_not_dict(self):
        with self.assertRaises(AttributeError):
            dotty(['not', 'valid'])

    def test_two_dotty_with_the_same_input_should_be_equal(self):
        first = dotty({'is': 'valid'})
        second = dotty({'is': 'valid'})

        self.assertEqual(first, second)

    def test_two_dotty_with_different_input_should_not_be_equal(self):
        first = dotty({'counter': 1})
        second = dotty({'counter': 2})

        self.assertNotEqual(first, second)

    def test_plain_dict_and_dotty_wrapper_should_be_equal(self):
        plain = {'a': 1, 'b': 2}
        dot = dotty(plain)
        self.assertEqual(dot, plain)

    def test_dotty_and_not_mapping_instance_should_not_be_equal(self):
        dot = dotty({'a': 1, 'b': 2})
        self.assertNotEqual(dot, [('a', 1), ('b', 2)])
        self.assertNotEqual(dot, ('a', 1))
        self.assertNotEqual(dot, {1, 2, 3})
        self.assertNotEqual(dot, 123)
        self.assertNotEqual(dot, 'a:1, b:2')

    def test_pop_with_default_value(self):
        dot = dotty()
        self.assertEqual(dot.pop('does.not.exist', None), None)
        self.assertEqual(dot.pop('does.not.exist', 55), 55)
        self.assertEqual(dot.pop('does.not.exist', 'my_value'), 'my_value')
